/*
 *  Branch.cpp
 *  Octopus
 *
 *  Created by Carson Baker on 7/18/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "Branch.h"

#import <Glut/glut.h>

#define TWO_PI 6.28318531
#define RAD_TO_DEG 57.2957795

int keypressed;

static void fghCircleTable(double **sint,double **cost,const int n)
{
    int i;
	
    /* Table size, the sign of n flips the circle direction */
	
    const int size = abs(n);
	
    /* Determine the angle between samples */
	
    const double angle = 2*M_PI/(double)( ( n == 0 ) ? 1 : n );
	
    /* Allocate memory for n samples, plus duplicate of first entry at the end */
	
    *sint = (double *) calloc(sizeof(double), size+1);
    *cost = (double *) calloc(sizeof(double), size+1);
	
    /* Bail out if memory allocation fails, fgError never returns */
	
    if (!(*sint) || !(*cost))
    {
        free(*sint);
        free(*cost);
    }
	
    /* Compute cos and sin around the circle */
	
    (*sint)[0] = 0.0;
    (*cost)[0] = 1.0;
	
    for (i=1; i<size; i++)
    {
        (*sint)[i] = sin(angle*i);
        (*cost)[i] = cos(angle*i);
    }
	
    /* Last sample is duplicate of the first */
	
    (*sint)[size] = (*sint)[0];
    (*cost)[size] = (*cost)[0];
}

/*
 * Draws a solid cylinder
 */
void glutSolidCylinder(GLdouble topRadius, GLdouble bottomRadius, GLdouble height, GLint slices, GLint stacks)
{
    int j;
	
    /* Pre-computed circle */
	
    double *sint,*cost;
	
    fghCircleTable(&sint,&cost,-slices);
	
    /* Cover the base and top */
	
//    glBegin(GL_TRIANGLE_FAN);
//	glNormal3d(0.0, 0.0, -1.0 );
//	glVertex3d(0.0, 0.0,  0.0 );
//	for (j=0; j<=slices; j++)
//		glVertex3d(cost[j]*bottomRadius, sint[j]*bottomRadius, 0.0);
//    glEnd();
//	
//    glBegin(GL_TRIANGLE_FAN);
//	glNormal3d(0.0, 0.0, 1.0   );
//	glVertex3d(0.0, 0.0, height);
//	for (j=slices; j>=0; j--)
//		glVertex3d(cost[j]*topRadius, sint[j]*topRadius, height);
//    glEnd();

		
	glBegin(GL_QUAD_STRIP);
	for (j=0; j<=slices; j++ )
	{
		glNormal3d(cost[j],        sint[j],        0.0 );
		glVertex3d(cost[j]*topRadius, 0, sint[j]*topRadius  );
		glVertex3d(cost[j]*topRadius, height, sint[j]*topRadius );
	}
	glEnd();

	
    /* Release sin and cos tables */
	
    free(sint);
    free(cost);
}


Branch::Branch(float angleX, float angleY)
{	
	_width			= 0;
	_height			= 0;
	child_nr		= 0;
	_angleX			= angleX; 
	_angleY			= angleY; 
	_angPosX		= 0; 
	_widthspeed		=.05;
	_heightspeed	=.3; 
	nextchild		= 2;
	
	_colorr = randfloat(.3,.5);
	_colorg = randfloat(.4,.7);
	_colorb = randfloat(.7,.9);
	
	_widthspeed=.05; 

	SyncTo(60); // must be less than game's master timer
	
	_height	+=_heightspeed; 
	_width	+=_widthspeed; 


}

void Branch::KeyDown(int a) {
	keypressed = 1;
}

void Branch::KeyUp(int a) {
	keypressed = 0;
}

void Branch::Draw() {

	float xrot = _angleX + sin(_angPosX+=.1/_width) / 10 + PI/2;
	float zrot = _angleY + sin(_angPosX+=.1/_width) / 10;
	
	xrot *= RAD_TO_DEG;
	zrot *= RAD_TO_DEG;

	glRotatef(xrot,0,1,0);
	glRotatef(zrot,0,0,1);
	glColor4f(_width/20,_colorg,_colorb,.5); 
	
	//gluCylinder(quadratic,_width/2.3,_width/2,3.0f,_height,12);
	//drawCylinder(_width/2, _width/2.3, _height, 12);
	
	//glRotatef(180,1,0,0); 
	glutSolidCylinder(_width/2, _width/2.3, _height, 12, 12);
	
	glTranslatef(0,_height,0); 

	for(int i = 0;i<child_nr;i++){ 
		glPushMatrix(); 
		children[i]->Draw();  
		glPopMatrix(); 
	} 

}

void Branch::Update() {
	

	for(int i = 0;i<child_nr;i++){ 
		children[i]->Update();
	} 

	_height	+=_heightspeed; 
	_width	+=_widthspeed; 
	
	if(child_nr<3 && randfloat(0,_width*10)/((child_nr+1)*(child_nr+1))>15){ 
	//if(_width>nextchild && child_nr<3){ 
		nextchild+=randfloat(1,10)*(child_nr+1); 
		_heightspeed*=randfloat(.2,.4); 
		newangle = randfloat(.2,.8); 
		children[child_nr++] = new Branch(randfloat(-newangle,newangle), randfloat(-newangle,newangle)); 
	} 

}

void Branch::TimerHit(int secs, int thou) {
	
	Timed::TimerHit(secs, thou);
	
	if(secs > 10) return;
	
	if(Timed::isSynced()) {
		
		Update();

	}
}
